import glob from 'glob'

import { dirname } from 'path'
import { promises as fs } from 'fs'
import config from 'lib/config'

export default async function copyHTML(node) {
  for (const key in node.children) {
    const child = node.children[key]

    if (child.isDirectory) await copyHTML(child)
    else {
      const { html } = child,
        pattern = child.destPath.replace('.html', '-*.js'),
        jsPaths = glob
          .sync(pattern)
          .map(path => path.replace(config.destDir, '')),
        scriptTags = jsPaths.map(path => `<script src="${path}"></script>`),
        packedHtml = `${html} ${scriptTags.join(' ')}`

      try {
        await fs.mkdir(dirname(child.destPath), { recursive: true })
        await fs.writeFile(child.destPath, packedHtml)
      } catch (e) {
        console.log('Error writing file: ', e)
        throw e
      }
    }
  }
}
