import matter from 'gray-matter'

import { promises as fs } from 'fs'
import config from 'lib/config'

function fullPath(node) {
  const fullpath = [node.name]

  while (node.parent?.name) {
    node = node.parent
    fullpath.unshift(node.name)
  }

  return '/' + fullpath.join('/')
}

class Node {
  children = {}

  constructor(opts) {
    Object.assign(this, opts)

    if (this.isRoot) this.isDirectory = true
    else this.path = fullPath(this)
  }

  add = async path => {
    const segments = typeof path == 'string' ? path.split('/') : path,
      nextSegment = segments[0]

    let node = this.children[nextSegment]

    if (!node) {
      node = new Node({ name: nextSegment, parent: this })
      this.children[nextSegment] = node
    }

    if (segments.length > 1) {
      node.isDirectory = true
      await node.add(segments.slice(1))
    } else {
      await node.change()
    }
  }

  get = path => {
    const segments = typeof path == 'string' ? path.split('/') : path

    return segments.reduce((node, segment) => {
      return node.children[segment]
    }, this)
  }

  remove = async () => {}

  change = async () => {
    const fileContents = (await fs.readFile(this.path)).toString(),
      { content, data } = matter(fileContents)

    this.source = content
    this.data = data
  }

  get isFile() {
    return !this.isDirectory
  }

  get relativePath() {
    return this.path.replace(config.srcDir, '')
  }

  get prepackPath() {
    return this.path
      .replace(/\.md$/, '.html')
      .replace(config.srcDir, config.prePackDir)
  }

  get destPath() {
    return this.path
      .replace(/\.md$/, '.html')
      .replace(config.srcDir, config.destDir)
  }

  get pageContext() {
    return {
      path: this.path,
    }
  }
}

export default new Node({ isRoot: true })
